/** \file Settings.cpp
*/

/* Copyright (C) 2008-2009 Tomasz Ostrowski <tomasz.o.ostrowski at gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2
 * as published by the Free Software Foundation.

 * Miniscope is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#pragma hdrstop

#include "Settings.h"
#include "display/Hsv.h"
#include "Base64.h"
#include <memory>
#include <IniFiles.hpp>

//---------------------------------------------------------------------------

#pragma package(smart_init)

const float Settings::fRecorderMinInterval = 0.01f;
const float Settings::fRecorderMaxInterval = 7200.0f;


bool Settings::Read(AnsiString asFileName)
{
	TIniFile *ini = NULL;
	TStrings *strings = NULL;
	try
	{
		ini = new TIniFile(asFileName);
		// main window startup position
		int maxX = GetSystemMetrics(SM_CXSCREEN);
		int maxY = GetSystemMetrics(SM_CYSCREEN);
		iWidth = ini->ReadInteger("GUI", "AppWidth", 550);
		iHeight = ini->ReadInteger("GUI", "AppHeight", 400);
		if (iWidth < 250 || iWidth > maxX + 20)
		{
			iWidth = 250;
		}
		if (iHeight < 200 || iHeight > maxY + 20)
		{
			iHeight = 200;
		}
		iPosX = ini->ReadInteger("GUI", "AppPositionX", 30);
		iPosY = ini->ReadInteger("GUI", "AppPositionY", 30);
		if (iPosX < 0)
			iPosX = 0;
		if (iPosY < 0)
			iPosY = 0;
		if (iPosX + iWidth > maxX)
			iPosX = maxX - iWidth;
		if (iPosY + iHeight > maxY)
			iPosY = maxY - iHeight;
		bAlwaysOnTop = ini->ReadBool("GUI", "AlwaysOnTop", false);
		bWindowMaximized = ini->ReadBool("GUI", "Maximized", false);
		iMaxUiLogLines = ini->ReadInteger("Application", "MaxUiLogLines", 1000);
		bLogToFile = ini->ReadBool("Application", "LogToFile", true);
		iDisplayRefreshFreq = ini->ReadInteger("Application", "DisplayRefreshFreq", 20);
		//iSendSuffix = ini->ReadInteger("Application", "SendSuffix", SENDSUFFIX_CRLF);
		asInputDeviceDllName = ini->ReadString("Application", "InputDeviceDllName", "");
		asPlotPictureExportDir = ini->ReadString("Application", "PlotPictureExportDir", "");
		asPlotDataExportDir = ini->ReadString("Application", "PlotDataExportDir", "");
		asLanguage = ini->ReadString("Application", "Language", "en");
		fRecorderInterval = ini->ReadFloat("Recorder", "Interval", 10.0);
		if (fRecorderInterval < fRecorderMinInterval || fRecorderInterval > fRecorderMaxInterval)
		{
			fRecorderInterval = 10.0;
		}

		GraphVisibility.bGridX = ini->ReadBool("GraphVisibility", "GridX", true);
		GraphVisibility.bGridY = ini->ReadBool("GraphVisibility", "GridY", true);
		GraphVisibility.bGridLabelsX = ini->ReadBool("GraphVisibility", "GridLabelsX", true);
		GraphVisibility.bGridLabelsY = ini->ReadBool("GraphVisibility", "GridLabelsY", false);
		GraphVisibility.bCursor = ini->ReadBool("GraphVisibility", "Cursor", true);
		GraphVisibility.bGndMarker = ini->ReadBool("GraphVisibility", "GndMarker", true);
		GraphVisibility.bTriggerPosMarker = ini->ReadBool("GraphVisibility", "TriggerPosMarker", true);
		GraphVisibility.bTriggerValMarker = ini->ReadBool("GraphVisibility", "TriggerValMarker", true);
		GraphVisibility.bRangeValues = ini->ReadBool("GraphVisibility", "RangeValues", true);
		GraphVisibility.bCursorValues = ini->ReadBool("GraphVisibility", "CursorValues", true);

		Graph.ToolboxTransparency = ini->ReadInteger("Graph", "ToolboxTransparency", 255);
		if (Graph.ToolboxTransparency < 30 || Graph.ToolboxTransparency > 255)
			Graph.ToolboxTransparency = 255;
		Graph.bSmoothTraceMoving = ini->ReadBool("Graph", "SmoothTraceMoving", true);

		AnsiString asTraceColors = ini->ReadString("GraphColors", "Traces", "");
		std::auto_ptr<TStringList>slist(new TStringList());
		slist->Clear();
		slist->Delimiter = ' ';
		slist->DelimitedText = asTraceColors;
		for (int i=0; i<slist->Count; i++)
		{
            GraphColors.traces.push_back(atol(slist->Strings[i].c_str()));
		}
#if 0
		if (GraphColors.traces.size() == 0)
		{
			// create default trace color set
			Hsv hsv;
			enum { CNT = 8 };
			for (int i=0; i<CNT; i++)
			{
				unsigned int color = hsv.GetUniformColor(CNT, 0, i);
				GraphColors.traces.push_back(color);
			}
		}
#endif

		Measurements.bVisible = ini->ReadBool("Measurements", "Visible", false);
		Measurements.iPosX = ini->ReadInteger("Measurements", "PosX", 40);
		Measurements.iPosY = ini->ReadInteger("Measurements", "PosY", 40);
		if (Measurements.iPosX < 0)
			Measurements.iPosX = 0;
		if (Measurements.iPosY < 0)
			Measurements.iPosY = 0;
		if (Measurements.iPosX + 40 > maxX)
			Measurements.iPosX = maxX - 40;
		if (Measurements.iPosY + 40 > maxY)
			Measurements.iPosY = maxY - 40;

		Ms3Recording.iSizeLimit = ini->ReadInteger("Ms3Recording", "SizeLimit", 100*1024);	// 100 MB
		if (Ms3Recording.iSizeLimit < 5*1024 || Ms3Recording.iSizeLimit > 2000*1024)
		{
        	Ms3Recording.iSizeLimit = 100*1024;
		}

		FFT.iSamplesLimit = ini->ReadInteger("FFT", "SamplesLimit", 4096);
		if (FFT.iSamplesLimit < 512 || FFT.iSamplesLimit > S_FFT::SAMPLES_HARD_LIMIT)
			FFT.iSamplesLimit = 4096;
		FFT.asWindowName = ini->ReadString("FFT", "Window", "Hanning");
		// read MRU items
		std::auto_ptr<TStringList> items(new(TStringList));
		items->StrictDelimiter = true;
		items->CommaText = ini->ReadString("MRUjson", "Files", "");
		MRUjson.clear();
		for (int i=0; i<std::min(items->Count, (int)MRU_LIMIT); i++)
		{
			AnsiString text = items->Strings[i];
			MRUjson.push_back(base64_decode(text.c_str()).c_str());
		}
		items->Clear();
		items->CommaText = ini->ReadString("MRUms3", "Files", "");
		MRUms3.clear();
		for (int i=0; i<std::min(items->Count, (int)MRU_LIMIT); i++)
		{
			AnsiString text = items->Strings[i];
			MRUms3.push_back(base64_decode(text.c_str()).c_str());
		}

		Zoom.bKeepRound = ini->ReadBool("Zoom", "KeepRound", false);
		Zoom.fZoomX = ini->ReadFloat("Zoom", "ZoomX", 1.0f);
		strings = new TStringList();
		ini->ReadSection("Zoom", strings);
		for (int i=0; ; i++)
		{
			AnsiString key;
			key.sprintf("ZoomY%d", i);
			if (strings->IndexOf(key) < 0)
				break;
			float val = ini->ReadFloat("Zoom", key, 1.0f);
			Zoom.fZoomY.push_back(val);
		}

		delete ini;
		delete strings;
	}
	catch (...)
	{
		if (ini)
		{
			delete ini;
		}
		if (strings)
		{
            delete strings;
        }
		return false;
	}
	return true;
}

bool Settings::Write(AnsiString asFileName)
{
	TIniFile *ini = NULL;
	try
	{
		ini = new TIniFile(asFileName);
		ini->WriteInteger("GUI", "AppWidth", iWidth);
		ini->WriteInteger("GUI", "AppHeight", iHeight);
		ini->WriteInteger("GUI", "AppPositionX", iPosX);
		ini->WriteInteger("GUI", "AppPositionY", iPosY);
		ini->WriteBool("GUI", "AlwaysOnTop", bAlwaysOnTop);
		ini->WriteBool("GUI", "Maximized", bWindowMaximized);	
		ini->WriteInteger("Application", "MaxUiLogLines", iMaxUiLogLines);
		ini->WriteBool("Application", "LogToFile", bLogToFile);
		ini->WriteInteger("Application", "DisplayRefreshFreq", iDisplayRefreshFreq);
		ini->WriteString("Application", "InputDeviceDllName", asInputDeviceDllName);
		ini->WriteString("Application", "PlotPictureExportDir", asPlotPictureExportDir);
		ini->WriteString("Application", "PlotDataExportDir", asPlotDataExportDir);
		ini->WriteString("Application", "Language", asLanguage);

		ini->WriteFloat("Recorder", "Interval", fRecorderInterval);

		ini->WriteBool("GraphVisibility", "GridX", GraphVisibility.bGridX);
		ini->WriteBool("GraphVisibility", "GridY", GraphVisibility.bGridY);
		ini->WriteBool("GraphVisibility", "GridLabelsX", GraphVisibility.bGridLabelsX);
		ini->WriteBool("GraphVisibility", "GridLabelsY", GraphVisibility.bGridLabelsY);
		ini->WriteBool("GraphVisibility", "Cursor", GraphVisibility.bCursor);
		ini->WriteBool("GraphVisibility", "GndMarker", GraphVisibility.bGndMarker);
		ini->WriteBool("GraphVisibility", "TriggerPosMarker", GraphVisibility.bTriggerPosMarker);
		ini->WriteBool("GraphVisibility", "TriggerValMarker", GraphVisibility.bTriggerValMarker);
		ini->WriteBool("GraphVisibility", "RangeValues", GraphVisibility.bRangeValues);
		ini->WriteBool("GraphVisibility", "CursorValues", GraphVisibility.bCursorValues);


		ini->WriteInteger("Graph", "ToolboxTransparency", Graph.ToolboxTransparency);
		ini->WriteBool("Graph", "SmoothTraceMoving", Graph.bSmoothTraceMoving);

		ini->WriteBool("Zoom", "KeepRound", Zoom.bKeepRound);
		ini->WriteFloat("Zoom", "ZoomX", Zoom.fZoomX);
		for (unsigned int i=0; i<Zoom.fZoomY.size(); i++)
		{
			AnsiString key;
			key.sprintf("ZoomY%d", i);
			ini->WriteFloat("Zoom", key, Zoom.GetY(i));
		}

		std::auto_ptr<TStringList>slist(new TStringList());
		slist->Clear();
		slist->Delimiter = ' ';
		for (unsigned int i=0; i<GraphColors.traces.size(); i++)
		{
			AnsiString val;
			val.sprintf("%u", GraphColors.traces[i]);
			slist->Add(val);
		}
		ini->WriteString("GraphColors", "Traces", slist->DelimitedText);

		ini->WriteBool("Measurements", "Visible", Measurements.bVisible);
		ini->WriteInteger("Measurements", "PosX", Measurements.iPosX);
		ini->WriteInteger("Measurements", "PosY", Measurements.iPosY);

		ini->WriteInteger("Ms3Recording", "SizeLimit", Ms3Recording.iSizeLimit);

		ini->WriteInteger("FFT", "SamplesLimit", FFT.iSamplesLimit);
		ini->WriteString("FFT", "Window", FFT.asWindowName);
		// write MRU items
		std::auto_ptr<TStringList> items(new(TStringList));
		items->StrictDelimiter = true;
		for (unsigned int i=0; i<MRUjson.size(); i++)
		{
			AnsiString file = MRUjson[i];
			items->Add(base64_encode((unsigned char*)file.c_str(), file.Length() + 1).c_str());
		}
		ini->WriteString("MRUjson", "Files", items->CommaText);
		items->Clear();
		for (unsigned int i=0; i<MRUms3.size(); i++)
		{
			AnsiString file = MRUms3[i];
			items->Add(base64_encode((unsigned char*)file.c_str(), file.Length() + 1).c_str());
		}
		ini->WriteString("MRUms3", "Files", items->CommaText);

		delete ini;
	}
	catch (...)
	{
		if (ini)
		{
			delete ini;
		}
		return false;
	}
	return true;

}

void Settings::AddMruJson(AnsiString item)
{
	// check if item is already in MRU list
	std::deque<AnsiString>::iterator iter;
	for (iter=MRUjson.begin(); iter != MRUjson.end(); )
	{
		if (*iter == item)
		{
			iter = MRUjson.erase(iter);
		}
		else
		{
            ++iter;
        }
	}
	// push item on top
	MRUjson.push_back(item);
	while (MRUjson.size() > MRU_LIMIT)
	{
		MRUjson.pop_front();
    }
}

void Settings::ClearMruJsonItems(void)
{
	MRUjson.clear();
}

void Settings::AddMruMs3(AnsiString item)
{
	// check if item is already in MRU list
	std::deque<AnsiString>::iterator iter;
	for (iter=MRUms3.begin(); iter != MRUms3.end(); )
	{
		if (*iter == item)
		{
			iter = MRUms3.erase(iter);
		}
		else
		{
            ++iter;
        }
	}
	// push item on top
	MRUms3.push_back(item);
	while (MRUms3.size() > MRU_LIMIT)
	{
		MRUms3.pop_front();
    }
}

void Settings::ClearMruMs3Items(void)
{
	MRUms3.clear();
}

float Settings::S_ZOOM::GetY(int channel)
{
	if (channel < (int)fZoomY.size())
	{
		return fZoomY[channel];
	}
	else
	{
    	return 1.0f;
    }
}

void Settings::S_ZOOM::SetY(int channel, float val)
{
	for (int i=fZoomY.size(); i<=channel; i++)
	{
		fZoomY.push_back(1.0f);
	}
	fZoomY[channel] = val;
}
